/* actions.c:

   All the actions that wmcd understands

   */

#include <X11/Xlib.h>

#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include "xpm/msgs/mounted.xpm"
#include "xpm/msgs/busy.xpm"
#include "xpm/msgs/notaudio.xpm"
#include "xpm/msgs/nodata.xpm"
#include "xpm/msgs/startup.xpm"

#include "cdrom.h"
#include "config.h"

#include "typedefs.h"
#include "window.h"

extern Display *dpy;
extern Window theWin;

extern Cursor stopwatch;

extern XpmIcon wmcd, led;
extern XpmIcon message;
extern XpmIcon trayctl, trayctl_p,
  opentray, opentray_p,
  closetray, closetray_p;
extern XpmIcon music, data;

extern int show_led;
extern int track_playing;
extern cdrom_position_t position;


/* function prototypes */

void EvalState(void);


void TrayCtl(void)
{
  XDefineCursor(dpy, theWin, stopwatch);
  XFlush(dpy);
  if(cdrom_tray_open())
    {
#ifdef DEBUG
      fprintf(stderr, "[A: Closing tray]\n");
#endif
      if(cdrom_closetray() == -1)
	{
	  fprintf(stderr, "Couldn't close tray\n");
	  /*	  XUndefineCursor(dpy, theWin);
	  return;*/
	}
      trayctl = opentray;
      trayctl_p = opentray_p;

      if(cdrom_cd_in_drive())
	{
#ifdef DEBUG
	  fprintf(stderr, "++ loaded ");
#endif
	  if(cdrom_cd_is_data_only())
	    {
#ifdef DEBUG
	      fprintf(stderr, "data-only CD\n");
#endif
	      if(cdrom_mount() != -1) /* i.e. mounting not disabled */
		{
		  CreateMessage(mounted_xpm);
		  show_led = False;
		}
	    }
	  else
	    {
#ifdef DEBUG
	      fprintf(stderr, "audio or mixed-mode CD\n");
#endif
	      track_playing = cdrom_no_of_tracks();
	      cdrom_length_of_cd(&position);
	      show_led = True;
	      DrawLED();
	      XSync(dpy, True);
	      
	      sleep(3);
	      
	      XSync(dpy, True);
	      track_playing = cdrom_current_track();
	      cdrom_get_position_relative(&position);
	    }
	} /* if cd_in_drive */
	
      EvalState();
      UpdateShape();
      RedrawWindow();
    }
  else /* if(!cdrom_tray_open()) */
    {

#ifdef DEBUG
      fprintf(stderr, "[A: Opening tray]\n");
#endif
      
      if(cdrom_mounted())
	if(cdrom_umount() == -1)
	  {
	    if(errno == EBUSY)
	      {
		XUndefineCursor(dpy, theWin);
		CreateMessage(busy_xpm);
		show_led = False;
		RedrawWindow();
		XSync(dpy, True);
		
		sleep(3);
		
		XSync(dpy, True);
		CreateMessage(mounted_xpm);
#ifdef DEBUG
		fprintf(stderr, "++ busy\n");
		fprintf(stderr, "[Done]\n");
#endif
		return;
	      }
	    else
	      {
		perror("TrayCtl(): umount()");
		return;
	      }
	  }


      if(cdrom_opentray() == -1)
	{
	  fprintf(stderr, "Couldn't open tray\n");
	  /*	  XUndefineCursor(dpy, theWin);
	  return;*/
	}
      trayctl = closetray;
      trayctl_p = closetray_p;
      cdrom_update_state();
      EvalState();
      UpdateShape();
      CreateMessage(startup_xpm);
      show_led = False;
      RedrawWindow();
    }
  XUndefineCursor(dpy, theWin);

#ifdef DEBUG
  fprintf(stderr, "[Done]\n");
#endif
}

void TrayMount(void)
{
#ifdef DEBUG
  fprintf(stderr, "[A: TrayMount]\n");
#endif
  
  XDefineCursor(dpy, theWin, stopwatch);
  XSync(dpy, True);
  if(cdrom_mounted())
    {
#ifdef DEBUG
      fprintf(stderr, "++ CD-ROM is mounted; umounting first\n");
#endif

      if(cdrom_umount() == -1)
	if(errno == EBUSY)
	  {
#ifdef DEBUG
	    fprintf(stderr, "++ busy\n");
#endif
	    CreateMessage(busy_xpm);
	    show_led = False;
	    RedrawWindow();
	    XSync(dpy, True);
	    sleep(3);
	    CreateMessage(mounted_xpm);
	    RedrawWindow();
	    XSync(dpy, True);
	  }
	else
	  perror("wmcd");
      else
	{
	  /* STOP it so it doesn't display obsolete information */
	  if(cdrom_stop() == -1)
	    {
	      perror("wmcd");
	      exit(1);
	    }
	  show_led = True;
	  EvalState();
	  UpdateShape();
	}
      
    }
  else /* if !mounted */
    {
      if(cdrom_tray_open()) /* Close it */
	{
#ifdef DEBUG
	  fprintf(stderr, "++ closing tray\n");
#endif
	  XDefineCursor(dpy, theWin, stopwatch);
	  XSync(dpy, True);
	  if(cdrom_closetray() == -1)
	    {
	      perror("wmcd");
	      exit(1);
	    }
	  XSync(dpy, True);
	  XUndefineCursor(dpy, theWin);
	}
      else if(cdrom_playing()) /* stop */
	{
#ifdef DEBUG
	  fprintf(stderr, "++ stopping\n");
#endif
	  if(cdrom_stop() == -1)
	    {
	      perror("wmcd");
	      exit(1);
	    }
	}

      cdrom_update_state(); /* in case we just closed the tray */
      if(!cdrom_cd_contains_data() || !cdrom_cd_in_drive()) /* then we don't want it */
	/* Note: This can't happen unless we just closed the tray. */
	{

#ifdef DEBUG
	  if(!cdrom_cd_contains_data())
	    fprintf(stderr, "++ tried to mount an audio-only CD-ROM\n");
	  if(!cdrom_cd_in_drive())
	    fprintf(stderr, "++ no cd in the drive\n");
#endif
	    
	  if(cdrom_opentray() == -1)
	    {
	      perror("wmcd");
	      exit(1);
	    }
	  /*	  tray_open = True;*/
	  CreateMessage(nodata_xpm);
	  show_led = False;
	  RedrawWindow();
	  XSync(dpy, True);
	  sleep(3);
	  XSync(dpy, True);
	  CreateMessage(startup_xpm);
	  XUndefineCursor(dpy, theWin);
	  XSync(dpy, True);
#ifdef DEBUG
	  fprintf(stderr, "[Done]\n");
#endif
	  return;
	}
      
      /* Still there? Then mount it */
      if(cdrom_mount() != -1) /* i.e. mounting not disabled */
	{
	  CreateMessage(mounted_xpm);
	  show_led = False;
	  EvalState();
	  UpdateShape();
	}
    }
  XUndefineCursor(dpy, theWin);
  XSync(dpy, True);

#ifdef DEBUG
  fprintf(stderr, "[Done]\n");
#endif
}




void Play(void)
{
#ifdef DEBUG
  fprintf(stderr, "[A: Play]\n");
#endif
  
  if(!cdrom_playing())
    {
      XDefineCursor(dpy, theWin, stopwatch);
      XSync(dpy, True);
    }

  if(cdrom_tray_open())
    {
#ifdef DEBUG
      fprintf(stderr, "++ closing tray\n");
#endif
      if(cdrom_closetray() == -1)
	{
	  fprintf(stderr, "Couldn't close tray\n");
	  /*	  XUndefineCursor(dpy, theWin);
	  return;*/
	}


      trayctl = opentray;
      trayctl_p = opentray_p;
      cdrom_update_state();
      EvalState();
      UpdateShape();
      if(!cdrom_cd_in_drive() || cdrom_cd_is_data_only())
	{

#ifdef DEBUG
	  if(!cdrom_cd_in_drive())
	    fprintf(stderr, "++ no cd in the drive\n");
	  if(cdrom_cd_is_data_only())
	    fprintf(stderr, "++ trying to play a data-only CD-ROM\n");
#endif

	  if(cdrom_opentray() == -1)
	    {
	      perror("wmcd");
	      exit(1);
	    }

	  trayctl = closetray;
	  trayctl_p = closetray_p;
	  cdrom_update_state();
	  EvalState();
	  UpdateShape();
	  CreateMessage(notaudio_xpm);
	  show_led = False;
	  RedrawWindow();
	  XSync(dpy, True);
	  sleep(3);
	  XSync(dpy, True);
	  CreateMessage(startup_xpm);
	  RedrawWindow();
	  XUndefineCursor(dpy, theWin);
	  XSync(dpy, True);
#ifdef DEBUG
	  fprintf(stderr, "[Done]\n");
#endif
	  return;
	}
    }
  if(cdrom_mounted())
    {
#ifdef DEBUG
      fprintf(stderr, "++ CD-ROM is mounted; umounting first\n");
#endif

      if(cdrom_umount() == -1)
	{
	  perror("wmcd");
	  XUndefineCursor(dpy, theWin);
	  XSync(dpy, True);

#ifdef DEBUG
	  fprintf(stderr, "[Done]\n");
#endif

	  return;
	}
      show_led = True;
      cdrom_update_state();
      EvalState();
      UpdateShape();
    }

  if(cdrom_cd_in_drive())
    {
      /* start playing */
#ifdef DEBUG
      fprintf(stderr, "++ starting play\n");
#endif

      if(cdrom_play() == -1)
	{
	  perror("wmcd");
	  exit(1);
	}

      show_led = True;
      DrawLED();
    }
  XUndefineCursor(dpy, theWin);
  XSync(dpy, True);

#ifdef DEBUG
  fprintf(stderr, "[Done]\n");
#endif
}

void Stop(void)
{
#ifdef DEBUG
  fprintf(stderr, "[A: Stop]\n");
#endif
	 
  XDefineCursor(dpy, theWin, stopwatch);
  XFlush(dpy);
  if(cdrom_stop() == -1)
    {
      perror("wmcd");
      exit(1);
    }
  XUndefineCursor(dpy, theWin);
  track_playing = -1;

#ifdef DEBUG
  fprintf(stderr, "[Done]\n");
#endif
}

void Pause(void)
{
#ifdef DEBUG
  fprintf(stderr, "[A: Pause]\n");
#endif
  
  if(!cdrom_playing())
    {
#ifdef DEBUG
      fprintf(stderr, "++ not playing\n");
      fprintf(stderr, "[Done]\n");
#endif
      return;
    }
  if(!cdrom_paused())
    {
#ifdef DEBUG
      fprintf(stderr, "++ pausing\n");
#endif
      XDefineCursor(dpy, theWin, stopwatch);
      XFlush(dpy);
      if(cdrom_pause() == -1)
	{
	  perror("wmcd");
	  exit(1);
	}
      XUndefineCursor(dpy, theWin);
    }
  else
    {
#ifdef DEBUG
      fprintf(stderr, "++ un-pausing\n");
#endif
      XDefineCursor(dpy, theWin, stopwatch);
      XFlush(dpy);
      if(cdrom_pause() == -1)
	{
	  perror("wmcd");
	  exit(1);
	}
      XUndefineCursor(dpy, theWin);
    }
#ifdef DEBUG
  fprintf(stderr, "[Done]\n");
#endif
}

void SkipF(void)
{
#ifdef DEBUG
  fprintf(stderr, "[A: Skip Forward]\n");
#endif
  show_led = True;

  cdrom_skipf_audio();
  track_playing = cdrom_current_track();
#ifdef DEBUG
  fprintf(stderr, "[Done]\n");
#endif
}

void SkipB(void)
{
#ifdef DEBUG
  fprintf(stderr, "[A: Skip Backward]\n");
#endif
  show_led = True;

  cdrom_skipb_audio();
  track_playing = cdrom_current_track();
#ifdef DEBUG
  fprintf(stderr, "[Done]\n");
#endif
}

void Ffwd(void)
{
  if(!cdrom_playing())
    return;

  cdrom_skip_frames(SKIPFRAMES);
  track_playing = cdrom_current_track();
}

void Rev(void)
{
  if(!cdrom_playing())
    return;

  cdrom_skip_frames(-SKIPFRAMES);
  track_playing = cdrom_current_track();
}

